@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: acct.c,v 1.19 2000/12/07 15:31:57 gray Exp $";
+"@(#) $Id: acct.c,v 1.4 2001/01/26 11:48:12 gray Exp $";
 #endif
 #define LOG_EMPTY_USERNAME
 
@@ -54,9 +54,6 @@ static char rcsid[] =
 # include <radsql.h>
 #endif
 
-/*
- *	FIXME: this should be configurable.
- */
 int	doradwtmp = 1;
 
 static char porttypes[] = "ASITX";
@@ -343,18 +340,6 @@ rad_accounting_new(authreq, dowtmp)
 		rb_record = 1;
 	} while (0);  /* hack to make break in the above code work */
 
-#ifdef NT_DOMAIN_HACK
-        if (!rb_record && vp) {
-		char buffer[AUTH_STRING_LEN];
-		char *ptr;
-		if ((ptr = strchr(vp->strvalue, '\\')) != NULL) {
-			strncpy(buffer, ptr + 1, sizeof(buffer));
-			buffer[sizeof(buffer) - 1] = 0;
-			strcpy(vp->strvalue, buffer);
-		}
-	}
-#endif
-
 	/*
 	 *	Add any specific attributes for this username.
 	 */
@@ -453,14 +438,16 @@ rad_accounting_new(authreq, dowtmp)
 	 *	See if this was a portmaster reboot.
 	 */
 	if (status == DV_ACCT_STATUS_TYPE_ACCOUNTING_ON && nas_address) {
-		radlog(L_INFO, _("NAS %s restarted (Accounting-On packet seen)"),
+		radlog(L_NOTICE, 
+			_("NAS %s restarted (Accounting-On packet seen)"),
 			nas_name(nas_address));
 		radzap(nas_address, -1, NULL, ut.time);
 		write_nas_restart(status, ut.nas_address);
 		return 0;
 	}
 	if (status == DV_ACCT_STATUS_TYPE_ACCOUNTING_OFF && nas_address) {
-		radlog(L_INFO, _("NAS %s rebooted (Accounting-Off packet seen)"),
+		radlog(L_NOTICE, 
+			_("NAS %s rebooted (Accounting-Off packet seen)"),
 			nas_name(nas_address));
 		radzap(nas_address, -1, NULL, ut.time);
 		write_nas_restart(status, ut.nas_address);
@@ -846,7 +833,8 @@ rad_accounting(authreq, activefd)
 	}
 
 	huntgroup_access(authreq);
-	
+
+#if defined(PW_ASCEND_EVENT_REQUEST) && defined(PW_ASCEND_EVENT_RESPONSE)
 	/* Special handling for Ascend-Event-Request
 	 */
 	if (authreq->code == PW_ASCEND_EVENT_REQUEST) {
@@ -856,7 +844,8 @@ rad_accounting(authreq, activefd)
 		stat_inc(acct, authreq->ipaddr, num_resp);
 		return 0;
 	}
-
+#endif
+	
 	if (rad_accounting_new(authreq, doradwtmp) == 0 &&
 	    rad_accounting_orig(authreq, auth, NULL) == 0) {
 		/*
@@ -871,6 +860,28 @@ rad_accounting(authreq, activefd)
 	return -1;
 }
 
+void
+rad_acct_xmit(type, code, data, fd)
+	int type;
+	int code;
+	void *data;
+	int fd;
+{
+	AUTH_REQ *req = (AUTH_REQ*)data;
+	
+	if (code == 0) {
+		rad_send_reply(PW_ACCOUNTING_RESPONSE, req, NULL, NULL, fd);
+		radlog(L_NOTICE,
+		       _("Retransmitting ACCT reply: client %s, ID: %d"),
+		       client_name(req->ipaddr),
+		       req->id);
+	} else {
+		radlog(L_NOTICE,
+		       _("Dropping ACCT packet: client %s, ID: %d"),
+		       client_name(req->ipaddr),
+		       req->id);
+	}
+}
 
 /*
  *	Timeout handler (10 secs)
@@ -928,7 +939,7 @@ check_ts(ut)
 	char	address[DOTTED_QUAD_LEN];
 	char	port[32];
 	char	session_id[RUT_IDSIZE+1];
-	void	(*handler)(int);
+	RETSIGTYPE (*handler)(int);
 
 	/*
 	 *	Find NAS type.
