@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radutil.c,v 1.7 2000/12/07 15:32:00 gray Exp $"; 
+"@(#) $Id: radutil.c,v 1.3 2001/01/22 12:33:38 gray Exp $"; 
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -146,7 +146,7 @@ attrno_to_str(buf, request, attr_no)
 	VALUE_PAIR *request;
 	int attr_no;
 {
-	return attr_to_str(buf, request, dict_attrget(attr_no));
+	return attr_to_str(buf, request, attr_number_to_dict(attr_no));
 }
 
 static DICT_ATTR *
@@ -157,7 +157,7 @@ parse_dict_attr(p, endp)
 	char namebuf[MAX_DICTNAME];
 	
 	if (isdigit(*p)) {
-		return dict_attrget(strtol(p, endp, 10));
+		return attr_number_to_dict(strtol(p, endp, 10));
 	}
 	if (*p == '{') {
 		int len;
@@ -170,7 +170,7 @@ parse_dict_attr(p, endp)
 			return NULL;
 		strncpy(namebuf, p+1, len);
 		namebuf[len] = 0;
-		return dict_attrfind(namebuf);
+		return attr_name_to_dict(namebuf);
 	}
 	*endp = p;
 	return NULL;
