@@ -18,7 +18,7 @@
  */
 
 static char rcsid[] = 
-"$Id: util.c,v 1.11 2000/12/18 17:57:12 gray Exp $";
+"$Id: util.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -176,7 +176,7 @@ create_pair(attr, length, strval, lval)
 	VALUE_PAIR *pair;
 	DICT_ATTR  *dict;
 
-	dict = dict_attrget(attr);
+	dict = attr_number_to_dict(attr);
 	if (!dict) {
 		radlog(L_ERR, _("make_pair(): dictionary attr %d not found"),
 		       attr);
