@@ -18,7 +18,7 @@
  */
 
 static char rcsid[] = 
-"$Id: radwho.c,v 1.6 2000/11/03 18:30:06 gray Exp $";
+"$Id: radwho.c,v 1.2 2000/12/29 11:39:59 gray Exp $";
 
 #if defined(HAVE_CONFIG_H)
 # include <config.h>
@@ -269,7 +269,8 @@ NAS  *naslist;            /* List of known NASes */
 	        "date:27,"\
 	        "nas:32,"\
 	        "clid:17,"\
-	        "time:7"
+	        "time:7,"\
+		"ip:16:Location"
 
 	       
 char *fmtspec = NULL;
